@@ -18,14 +18,14 @@ from django.utils.text import slugify
 from allianceauth.authentication.models import State, get_guest_state,\
     CharacterOwnership, UserProfile, OwnershipRecord
 from allianceauth.hooks import get_hooks
-from allianceauth.eveonline.models import EveCharacter, EveCorporationInfo
+from allianceauth.eveonline.models import EveCharacter, EveCorporationInfo,\
+    EveAllianceInfo
 from allianceauth.eveonline.tasks import update_character
 from .app_settings import AUTHENTICATION_ADMIN_USERS_MAX_GROUPS, \
     AUTHENTICATION_ADMIN_USERS_MAX_CHARS
 
 if 'allianceauth.eveonline.autogroups' in settings.INSTALLED_APPS:
-    _has_auto_groups = True
-    from allianceauth.eveonline.autogroups.models import *
+    _has_auto_groups = True    
 else:
     _has_auto_groups = False
 
@@ -241,6 +241,22 @@ class MainAllianceFilter(admin.SimpleListFilter):
                         self.value())
                 
 
+def update_main_character_model(modeladmin, request, queryset):    
+    tasks_count = 0
+    for obj in queryset:
+        if obj.profile.main_character:
+            update_character.delay(obj.profile.main_character.character_id)
+            tasks_count += 1
+
+    modeladmin.message_user(
+        request, 
+        'Update from ESI started for {} characters'.format(tasks_count)
+    )
+
+update_main_character_model.short_description = \
+    'Update main character model from ESI'
+
+
 class UserAdmin(BaseUserAdmin):
     """Extending Django's UserAdmin model
     
@@ -272,28 +288,13 @@ class UserAdmin(BaseUserAdmin):
             else:    
                 return queryset.filter(groups__pk=self.value())
 
-    def update_main_character_model(self, request, queryset):    
-        tasks_count = 0
-        for obj in queryset:
-            if obj.profile.main_character:
-                update_character.delay(obj.profile.main_character.character_id)
-                tasks_count += 1
-
-        self.message_user(
-            request, 
-            'Update from ESI started for {} characters'.format(tasks_count)
-        )
-
-    update_main_character_model.short_description = \
-        'Update main character model from ESI'
-
     def get_actions(self, request):
         actions = super(BaseUserAdmin, self).get_actions(request)
 
-        actions[self.update_main_character_model.__name__] = (
-            self.update_main_character_model, 
-            self.update_main_character_model.__name__, 
-            self.update_main_character_model.short_description
+        actions[update_main_character_model.__name__] = (
+            update_main_character_model, 
+            update_main_character_model.__name__, 
+            update_main_character_model.short_description
         )
 
         for hook in get_hooks('services_hook'):
@@ -505,7 +506,7 @@ class BaseOwnershipAdmin(admin.ModelAdmin):
         'character',
     )
     search_fields = (
-        'user__user', 
+        'user__username', 
         'character__character_name', 
         'character__corporation_name', 
         'character__alliance_name'
