@@ -22,13 +22,6 @@ urlpatterns = [
         r'^account/characters/add/$', 
         views.add_character, 
         name='add_character'
-    ),
-    url(
-        r'^help/$', 
-        login_required(
-            TemplateView.as_view(template_name='allianceauth/help.html')
-        ),
-        name='help'
-    ),
+    ),   
     url(r'^dashboard/$', views.dashboard, name='dashboard'),
 ]
