@@ -6,9 +6,8 @@ from ...admin import ServicesUserAdmin
 
 @admin.register(DiscordUser)
 class DiscordUserAdmin(ServicesUserAdmin):            
-    list_display = ServicesUserAdmin.list_display + (        
-        '_uid',        
-    )    
+    list_display = ServicesUserAdmin.list_display + ('_uid',)
+    search_fields = ServicesUserAdmin.search_fields + ('uid', )
    
     def _uid(self, obj):
         return obj.uid
