@@ -8,7 +8,7 @@ from django.contrib.humanize.templatetags.humanize import intcomma
 from django.http import JsonResponse, Http404
 from django.shortcuts import render, redirect, get_object_or_404
 from django.utils import timezone
-from django.utils.translation import ugettext_lazy as _
+from django.utils.translation import gettext_lazy as _
 from django.db.models import Sum
 from allianceauth.authentication.decorators import permissions_required
 from allianceauth.eveonline.providers import provider
